/* eslint-disable no-shadow */
import React, { useImperativeHandle, useState } from 'react';
import { FlatList, TouchableOpacity, View, Text } from 'react-native';
import Ionicons from 'react-native-vector-icons/Ionicons';
import { styles } from './styles';
import _ from 'lodash';
const defaultProps = {
  style: {},
  textStyle: {},
  iconColor: 'black'
};
let selectItem = [];
let defaultValue = [];
const CheckboxTreeComponent = /*#__PURE__*/React.forwardRef((props, ref) => {
  const {
    data,
    keyField,
    textField,
    childField,
    style,
    textStyle,
    iconColor = 'black',
    iconSize = 26,
    openIcon,
    closeIcon,
    checkIcon,
    unCheckIcon,
    autoSelectParents = true,
    autoSelectChilds = true,
    renderItem
  } = props;
  const [listData, setListData] = useState(_.cloneDeep(data));
  const [key, setKey] = useState(Math.random());
  useImperativeHandle(ref, () => {
    return {
      clear,
      setSelectedItem,
      updateListData
    };
  });

  const clear = () => {
    onClear(listData);
  };

  const updateListData = data => {
    const tmpData = _.cloneDeep(data);

    setListData(tmpData);
    onDefault(tmpData, false);
  };

  const onClear = items => {
    items.map(item => {
      item.tick = false;
      item.show = false;

      if (item[childField]) {
        onClear(item[childField]);
      }
    });
    reload();
  };

  const setSelectedItem = data => {
    defaultValue = data;
    onDefault(listData, false);
  };

  const onDefault = function (items) {
    let tick = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
    items.map(item => {
      const check = _.filter(defaultValue, e => e === item[keyField]).length > 0;

      if (tick) {
        item.tick = true;
      } else {
        if (check) {
          item.tick = true;
        }
      }

      if (item[childField]) {
        onDefault(item[childField], autoSelectChilds && item.tick);
      }
    });
    reload();
  };

  const parent = item => {
    if (item && item[childField]) {
      const check = item[childField].filter(child => !child.tick);

      if (check.length === 0) {
        item.tick = true;
      } else {
        item.tick = false;
      }

      parent(item.parent);
      reload();
    }
  };

  const onTick = item => {
    item.tick = true;

    if (autoSelectParents) {
      parent(item.parent);
    }

    if (item[childField] && autoSelectChilds) {
      item[childField].map(child => onTick(child));
    }

    reload();
  };

  const onUnTick = item => {
    item.tick = false;

    if (autoSelectParents) {
      parent(item.parent);
    }

    if (item[childField] && autoSelectChilds) {
      item[childField].map(child => onUnTick(child));
    }

    reload();
  };

  const showChild = item => {
    item.show = !item.show;
    reload();
  };

  const onSelectItem = () => {
    const selectedItem = _.cloneDeep(selectItem).map(e => {
      delete e.parent;
      delete e.childs;
      delete e.show;
      delete e.tick;
      return e;
    });

    props.onSelect(selectedItem);
  };

  const reload = _.throttle(() => {
    setKey(Math.random());
    selectItem = [];
    selectItemTick(listData);
    onSelectItem();
  }, 500);

  const selectItemTick = data => {
    data.map(item => {
      if (item.tick) {
        selectItem.push(item);
      }

      if (item[childField]) {
        selectItemTick(item[childField]);
      }
    });
  };

  const _renderIcon = status => {
    if (status) {
      if (checkIcon) {
        return checkIcon;
      } else {
        return /*#__PURE__*/React.createElement(Ionicons, {
          name: "ios-checkbox-outline",
          size: iconSize,
          color: iconColor
        });
      }
    } else {
      if (unCheckIcon) {
        return unCheckIcon;
      } else {
        return /*#__PURE__*/React.createElement(Ionicons, {
          name: "stop-outline",
          size: iconSize,
          color: iconColor
        });
      }
    }
  };

  const renderList = (item, childs, index) => {
    if (!item.show) {
      item.show = false;
    }

    if (!item.tick) {
      item.tick = false;
    }

    return /*#__PURE__*/React.createElement(View, {
      style: [styles.item, {
        marginLeft: iconSize
      }],
      key: index
    }, renderItem ? renderItem({
      item: item,
      isSelect: item.tick,
      isOpen: item.show,
      onOpen: () => {
        showChild(item);
      },
      onClose: () => {
        showChild(item);
      },
      onSelect: () => {
        !item.tick ? onTick(item) : onUnTick(item);
      }
    }) : /*#__PURE__*/React.createElement(View, {
      style: styles.rowItem
    }, childs && childs.length > 0 ? /*#__PURE__*/React.createElement(TouchableOpacity, {
      onPress: () => {
        showChild(item);
      }
    }, item.show ? openIcon ? openIcon : /*#__PURE__*/React.createElement(Ionicons, {
      name: "ios-remove",
      size: iconSize,
      color: iconColor
    }) : closeIcon ? closeIcon : /*#__PURE__*/React.createElement(Ionicons, {
      name: "add-outline",
      size: iconSize,
      color: iconColor
    })) : /*#__PURE__*/React.createElement(Text, {
      style: {
        width: iconSize
      }
    }, `  `), /*#__PURE__*/React.createElement(TouchableOpacity, {
      style: styles.flex1,
      onPress: () => {
        if (!item.tick) {
          onTick(item);
        } else {
          onUnTick(item);
        }
      }
    }, /*#__PURE__*/React.createElement(View, {
      style: styles.center
    }, _renderIcon(item.tick), /*#__PURE__*/React.createElement(Text, {
      style: [styles.name, textStyle],
      numberOfLines: 3
    }, item[textField])))), /*#__PURE__*/React.createElement(View, {
      style: [!item.show && styles.height0]
    }, childs && childs.map((data, index) => {
      if (!data.parent) {
        data.parent = item;
      }

      return renderList(data, data[childField], index);
    })));
  };

  return /*#__PURE__*/React.createElement(View, {
    style: [styles.container, style]
  }, /*#__PURE__*/React.createElement(FlatList, {
    data: listData,
    renderItem: _ref => {
      let {
        item,
        index
      } = _ref;
      return renderList(item, item[childField], index);
    },
    keyExtractor: (_item, index) => index.toString(),
    showsVerticalScrollIndicator: false,
    extraData: key
  }));
});
CheckboxTreeComponent.defaultProps = defaultProps;
export default CheckboxTreeComponent;
//# sourceMappingURL=index.js.map